@@ -275,7 +275,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF
+#if !CF && !__MonoCS__
 
         protected override DbProviderFactory DbProviderFactory
         {
@@ -284,6 +284,7 @@ namespace MySql.Data.MySqlClient
                 return MySqlClientFactory.Instance;
             }
         }
+		
 #endif
 
         #endregion
@@ -521,7 +522,7 @@ namespace MySql.Data.MySqlClient
         /// Creates a new MySqlConnection object with the exact same ConnectionString value
         /// </summary>
         /// <returns>A cloned MySqlConnection object</returns>
-        object ICloneable.Clone()
+        public MySqlConnection Clone()
         {
             MySqlConnection clone = new MySqlConnection();
             string connectionString = settings.GetConnectionString(true);
@@ -530,6 +531,11 @@ namespace MySql.Data.MySqlClient
             return clone;
         }
 
+        object ICloneable.Clone()
+        {
+            return this.Clone();
+        }
+
         #endregion
 
         #region IDisposeable
