@@ -42,10 +42,11 @@ namespace MySql.Data.MySqlClient
         uint procCacheSize, connectionLifetime;
         MySqlConnectionProtocol protocol;
         MySqlDriverType driverType;
+        MySqlSslMode sslMode;
         bool compress, connectionReset, allowBatch, logging;
         bool oldSyntax, persistSI, usePerfMon, pooling;
         bool allowZeroDatetime, convertZeroDatetime;
-        bool useUsageAdvisor, useSSL;
+        bool useUsageAdvisor;
         bool ignorePrepare, useProcedureBodies;
         bool autoEnlist, respectBinaryFlags, treatBlobsAsUTF8;
         string blobAsUtf8IncludePattern, blobAsUtf8ExcludePattern;
@@ -57,6 +58,8 @@ namespace MySql.Data.MySqlClient
         bool interactiveSession;
         bool functionsReturnString;
         bool useAffectedRows;
+        bool oldGuids;
+        uint keepalive;
 
         static MySqlConnectionStringBuilder()
         {
@@ -86,7 +89,7 @@ namespace MySql.Data.MySqlClient
             defaultValues.Add(Keyword.AllowZeroDatetime, false);
             defaultValues.Add(Keyword.UsePerformanceMonitor, false);
             defaultValues.Add(Keyword.ProcedureCacheSize, 25);
-            defaultValues.Add(Keyword.UseSSL, false);
+            defaultValues.Add(Keyword.Encrypt, false);
             defaultValues.Add(Keyword.IgnorePrepare, true);
             defaultValues.Add(Keyword.UseProcedureBodies, true);
             defaultValues.Add(Keyword.AutoEnlist, true);
@@ -100,6 +103,9 @@ namespace MySql.Data.MySqlClient
             defaultValues.Add(Keyword.InteractiveSession, false);
             defaultValues.Add(Keyword.FunctionsReturnString, false);
             defaultValues.Add(Keyword.UseAffectedRows, false);
+            defaultValues.Add(Keyword.SslMode, MySqlSslMode.None);
+            defaultValues.Add(Keyword.OldGuids, false);
+            defaultValues.Add(Keyword.Keepalive,0);
         }
 
         /// <summary>
@@ -134,7 +140,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets the name of the server.
         /// </summary>
         /// <value>The server.</value>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [Description("Server to connect to")]
         [RefreshProperties(RefreshProperties.All)]
@@ -153,7 +159,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets the name of the database the connection should 
         /// initially connect to.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [Description("Database to use initially")]
         [RefreshProperties(RefreshProperties.All)]
@@ -172,7 +178,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets the protocol that should be used for communicating
         /// with MySQL.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Connection Protocol")]
         [Description("Protocol to use for connection to MySQL")]
@@ -193,7 +199,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets the name of the named pipe that should be used
         /// for communicating with MySQL.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Pipe Name")]
         [Description("Name of pipe to use when connecting with named pipes (Win32 only)")]
@@ -214,7 +220,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets a boolean value that indicates whether this connection
         /// should use compression.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Use Compression")]
         [Description("Should the connection ues compression")]
@@ -235,7 +241,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets a boolean value that indicates whether this connection will allow
         /// commands to send multiple SQL statements in one execution.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Allow Batch")]
         [Description("Allows execution of multiple SQL commands in a single statement")]
@@ -255,7 +261,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets a boolean value that indicates whether logging is enabled.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [Description("Enables output of diagnostic messages")]
         [DefaultValue(false)]
@@ -275,7 +281,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets the base name of the shared memory objects used to 
         /// communicate with MySQL when the shared memory protocol is being used.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Shared Memory Name")]
         [Description("Name of the shared memory object to use")]
@@ -296,7 +302,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets a boolean value that indicates whether this connection uses
         /// the old style (@) parameter markers or the new (?) style.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Use Old Syntax")]
         [Description("Allows the use of old style @ syntax for parameters")]
@@ -320,7 +326,7 @@ namespace MySql.Data.MySqlClient
         /// <remarks>
         /// There is only one valid value for this setting currently.
         /// </remarks>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Driver Type")]
         [Description("Specifies the type of driver to use for this connection")]
@@ -342,7 +348,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets the port number that is used when the socket
         /// protocol is being used.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [Description("Port to use for TCP/IP connections")]
         [DefaultValue(3306)]
@@ -361,7 +367,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets the connection timeout.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Connect Timeout")]
         [Description("The length of time (in seconds) to wait for a connection " +
@@ -382,7 +388,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets the default command timeout.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Connection")]
         [DisplayName("Default Command Timeout")]
         [Description(@"The default timeout that MySqlCommand objects will use
@@ -408,7 +414,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets the user id that should be used to connect with.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Security")]
         [DisplayName("User Id")]
         [Description("Indicates the user ID to be used when connecting to the data source.")]
@@ -427,7 +433,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets the password that should be used to connect with.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Security")]
         [Description("Indicates the password to be used when connecting to the data source.")]
         [PasswordPropertyText(true)]
@@ -447,7 +453,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets a boolean value that indicates if the password should be persisted
         /// in the connection string.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Security")]
         [DisplayName("Persist Security Info")]
         [Description("When false, security-sensitive information, such as the password, " +
@@ -465,19 +471,19 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Authentication")]
-        [Description("Should the connection use SSL.  This currently has no effect.")]
+        [Description("Should the connection use SSL.")]
         [DefaultValue(false)]
-        [RefreshProperties(RefreshProperties.All)]
+        [Obsolete("Use Ssl Mode instead.")]
 #endif
-            internal bool UseSSL
+        internal bool Encrypt
         {
-            get { return useSSL; }
+            get { return SslMode != MySqlSslMode.None; }
             set
             {
-                SetValue("UseSSL", value); 
-                useSSL = value;
+                SetValue("Encrypt", value);
+                sslMode = value ? MySqlSslMode.Prefered : MySqlSslMode.None;
             }
         }
 
@@ -488,7 +494,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets a boolean value that indicates if zero date time values are supported.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Allow Zero Datetime")]
         [Description("Should zero datetimes be supported")]
@@ -509,7 +515,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets a boolean value indicating if zero datetime values should be 
         /// converted to DateTime.MinValue.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Convert Zero Datetime")]
         [Description("Should illegal datetime values be converted to DateTime.MinValue")]
@@ -529,7 +535,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets a boolean value indicating if the Usage Advisor should be enabled.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Use Usage Advisor")]
         [Description("Logs inefficient database operations")]
@@ -549,7 +555,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets the size of the stored procedure cache.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Procedure Cache Size")]
         [Description("Indicates how many stored procedures can be cached at one time. " +
@@ -570,7 +576,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets a boolean value indicating if the permon hooks should be enabled.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Use Performance Monitor")]
         [Description("Indicates that performance counters should be updated during execution.")]
@@ -590,7 +596,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets a boolean value indicating if calls to Prepare() should be ignored.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Ignore Prepare")]
         [Description("Instructs the provider to ignore any attempts to prepare a command.")]
@@ -607,7 +613,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Use Procedure Bodies")]
         [Description("Indicates if stored procedure bodies will be available for parameter detection.")]
@@ -624,7 +630,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Auto Enlist")]
         [Description("Should the connetion automatically enlist in the active connection, if there are any.")]
@@ -641,7 +647,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Respect Binary Flags")]
         [Description("Should binary flags on column metadata be respected.")]
@@ -658,7 +664,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Treat Tiny As Boolean")]
         [Description("Should the provider treat TINYINT(1) columns as boolean.")]
@@ -675,7 +681,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Allow User Variables")]
         [Description("Should the provider expect user variables to appear in the SQL.")]
@@ -692,7 +698,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Interactive Session")]
         [Description("Should this session be considered interactive?")]
@@ -709,7 +715,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Functions Return String")]
         [Description("Should all server functions be treated as returning string?")]
@@ -725,7 +731,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         [Category("Advanced")]
         [DisplayName("Use Affected Rows")]
         [Description("Should the returned affected row count reflect affected rows instead of found rows?")]
@@ -741,6 +747,39 @@ namespace MySql.Data.MySqlClient
             }
         }
 
+
+#if !CF
+        [Category("Advanced")]
+        [DisplayName("Old Guids")]
+        [Description("Treat binary(16) columns as guids")]
+        [DefaultValue(false)]
+#endif
+        public bool OldGuids
+        {
+            get { return oldGuids; }
+            set 
+            {
+                SetValue("Old Guids", value);
+                oldGuids = value;
+            }
+        }
+
+#if !CF
+        [DisplayName("Keepalive")]
+        [Description("For TCP connections, idle connection time measured in seconds, before the first keepalive packet is sent." +
+            "A value of 0 indicates that keepalive is not used.")]
+        [DefaultValue(0)]
+#endif
+        public uint Keepalive
+        {
+            get { return keepalive;}
+            set 
+            {
+                SetValue("Keepalive", value);
+                keepalive = value;
+            }
+        }
+
         #endregion
 
         #region Pooling Properties
@@ -748,7 +787,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets the lifetime of a pooled connection.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Pooling")]
         [DisplayName("Connection Lifetime")]
         [Description("The minimum amount of time (in seconds) for this connection to " +
@@ -769,7 +808,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets a boolean value indicating if connection pooling is enabled.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Pooling")]
         [Description("When true, the connection object is drawn from the appropriate " +
                      "pool, or if necessary, is created and added to the appropriate pool.")]
@@ -789,7 +828,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets the minimum connection pool size.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Pooling")]
         [DisplayName("Minimum Pool Size")]
         [Description("The minimum number of connections allowed in the pool.")]
@@ -809,7 +848,7 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets the maximum connection pool setting.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Pooling")]
         [DisplayName("Maximum Pool Size")]
         [Description("The maximum number of connections allowed in the pool.")]
@@ -830,7 +869,7 @@ namespace MySql.Data.MySqlClient
         /// Gets or sets a boolean value indicating if the connection should be reset when retrieved
         /// from the pool.
         /// </summary>
-#if !CF && !MONO
+#if !CF 
         [Category("Pooling")]
         [DisplayName("Connection Reset")]
         [Description("When true, indicates the connection state is reset when " +
@@ -852,7 +891,7 @@ namespace MySql.Data.MySqlClient
 
         #region Language and Character Set Properties
 
-#if !CF && !MONO
+#if !CF 
         /// <summary>
         /// Gets or sets the character set that should be used for sending queries to the server.
         /// </summary>
@@ -871,7 +910,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         /// <summary>
         /// Indicates whether the driver should treat binary blobs as UTF8
         /// </summary>
@@ -890,7 +929,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         /// <summary>
         /// Gets or sets the pattern that matches the columns that should be treated as UTF8
         /// </summary>
@@ -910,7 +949,7 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-#if !CF && !MONO
+#if !CF 
         /// <summary>
         /// Gets or sets the pattern that matches the columns that should not be treated as UTF8
         /// </summary>
@@ -930,6 +969,24 @@ namespace MySql.Data.MySqlClient
             }
         }
 
+#if !CF
+        /// <summary>
+        /// Indicates whether to use SSL connections and how to handle server certificate errors.
+        /// </summary>
+        [DisplayName("SslMode")]
+        [Category("Security")]
+        [Description("SSL properties for connection")]
+#endif
+        public MySqlSslMode SslMode
+        {
+            get { return sslMode; }
+            set
+            {
+                SetValue("Ssl Mode", value);
+                sslMode = value;
+            }
+        }
+
         #endregion
 
         #region Conversion Routines
@@ -1002,6 +1059,13 @@ namespace MySql.Data.MySqlClient
                                          typeof (MySqlDriverType), value.ToString(), true);
         }
 
+        private static MySqlSslMode ConvertToSslMode(object value)
+        {
+            if (value is MySqlSslMode)
+                return (MySqlSslMode)value;
+            return (MySqlSslMode)Enum.Parse(typeof(MySqlSslMode), value.ToString(), true);
+        }
+
         #endregion
 
         #region Internal Properties
@@ -1148,7 +1212,7 @@ namespace MySql.Data.MySqlClient
                 case "IGNORE PREPARE":
                     return Keyword.IgnorePrepare;
                 case "ENCRYPT":
-                    return Keyword.UseSSL;
+                    return Keyword.Encrypt;
                 case "PROCEDURE BODIES":
                 case "USE PROCEDURE BODIES":
                     return Keyword.UseProcedureBodies;
@@ -1176,6 +1240,12 @@ namespace MySql.Data.MySqlClient
                     return Keyword.FunctionsReturnString;
                 case "USE AFFECTED ROWS":
                     return Keyword.UseAffectedRows;
+                case "SSL MODE":
+                    return Keyword.SslMode;
+                case "OLD GUIDS":
+                    return Keyword.OldGuids;
+                case "KEEPALIVE":
+                    return Keyword.Keepalive;
             }
             throw new ArgumentException(Resources.KeywordNotSupported, key);
         }
@@ -1238,8 +1308,8 @@ namespace MySql.Data.MySqlClient
                     return UsePerformanceMonitor;
                 case Keyword.IgnorePrepare:
                     return IgnorePrepare;
-                case Keyword.UseSSL:
-                    return UseSSL;
+                case Keyword.Encrypt:
+                    return Encrypt;
                 case Keyword.UseProcedureBodies:
                     return UseProcedureBodies;
                 case Keyword.AutoEnlist:
@@ -1264,6 +1334,12 @@ namespace MySql.Data.MySqlClient
                     return functionsReturnString;
                 case Keyword.UseAffectedRows:
                     return useAffectedRows;
+                case Keyword.SslMode:
+                    return sslMode;
+                case Keyword.OldGuids:
+                    return oldGuids;
+                case Keyword.Keepalive:
+                    return keepalive;
                 default:
                     return null; /* this will never happen */
             }
@@ -1349,8 +1425,11 @@ namespace MySql.Data.MySqlClient
                     procCacheSize = ConvertToUInt(value); break;
                 case Keyword.IgnorePrepare: 
                     ignorePrepare = ConvertToBool(value); break;
-                case Keyword.UseSSL: 
-                    useSSL = ConvertToBool(value); break;
+                case Keyword.Encrypt: 
+                    if (!clearing)
+                        Logger.LogWarning("Encrypt is now obsolete. Use Ssl Mode instead");
+                    sslMode = ConvertToBool(value)?MySqlSslMode.Prefered:MySqlSslMode.None;
+                    break;
                 case Keyword.UseProcedureBodies: 
                     useProcedureBodies = ConvertToBool(value); break;
                 case Keyword.AutoEnlist:
@@ -1375,6 +1454,12 @@ namespace MySql.Data.MySqlClient
                     functionsReturnString = ConvertToBool(value); break;
                 case Keyword.UseAffectedRows:
                     useAffectedRows = ConvertToBool(value); break;
+                case Keyword.SslMode:
+                    sslMode = ConvertToSslMode(value); break;
+                case Keyword.OldGuids:
+                    oldGuids = ConvertToBool(value); break;
+                case Keyword.Keepalive:
+                    keepalive = ConvertToUInt(value); break;
             }
         }
 
@@ -1545,7 +1630,7 @@ namespace MySql.Data.MySqlClient
         UsePerformanceMonitor,
         ProcedureCacheSize,
         IgnorePrepare,
-        UseSSL,
+        Encrypt,
         UseProcedureBodies,
         AutoEnlist,
         RespectBinaryFlags,
@@ -1557,6 +1642,9 @@ namespace MySql.Data.MySqlClient
         AllowUserVariables,
         InteractiveSession,
         FunctionsReturnString,
-        UseAffectedRows
+        UseAffectedRows,
+        SslMode,
+        OldGuids,
+        Keepalive
     }
 }
