@@ -95,36 +95,6 @@ namespace MySql.Data.MySqlClient
             return packet; 
         }
 
-        /// <summary>
-        /// ExecuteCommand does the work of writing the actual command bytes to the writer
-        /// We break it out into a function since it is used in several places besides query
-        /// </summary>
-        /// <param name="cmd">The cmd that we are sending</param>
-        /// <param name="bytes">The bytes of the command, can be null</param>
-        /// <param name="length">The number of bytes to send</param>
-/*        private void ExecuteCommand(DBCmd cmd, byte[] bytes, int length)
-        {
-            Debug.Assert(length == 0 || bytes != null);
-
-            try
-            {
-                stream.StartOutput((ulong) length + 1, true);
-                stream.WriteByte((byte) cmd);
-                if (length > 0)
-                    stream.Write(bytes, 0, length);
-                stream.Flush();
-            }
-            catch (MySqlException ex)
-            {
-                if (ex.IsFatal)
-                {
-                    isOpen = false;
-                    Close();
-                }
-                throw;
-            }
-        }*/
-
         private void ReadOk(bool read)
         {
             try
@@ -201,7 +171,8 @@ namespace MySql.Data.MySqlClient
                     string pipeName = Settings.PipeName;
                     if (Settings.ConnectionProtocol != MySqlConnectionProtocol.NamedPipe)
                         pipeName = null;
-                    StreamCreator sc = new StreamCreator(Settings.Server, Settings.Port, pipeName);
+                    StreamCreator sc = new StreamCreator(Settings.Server, Settings.Port, pipeName,
+                        Settings.Keepalive);
                     baseStream = sc.GetStream(Settings.ConnectionTimeout);
 #if !CF
                 }
@@ -254,12 +225,21 @@ namespace MySql.Data.MySqlClient
                                 version.isAtLeast(4, 1, 0) ? 4 : 2);
 
 #if !CF
-            if (connectionString.UseSSL && (serverCaps & ClientFlags.SSL) != 0)
+            if ((serverCaps & ClientFlags.SSL) ==0)
+            {
+                if ((connectionString.SslMode != MySqlSslMode.None)
+                && (connectionString.SslMode != MySqlSslMode.Prefered))
+                {
+                    // Client requires SSL connections.
+                    string message = String.Format(Resources.NoServerSSLSupport,
+                        Settings.Server);
+                    throw new MySqlException(message);
+                }
+            }
+            else if (connectionString.SslMode != MySqlSslMode.None)
             {
                 stream.SendPacket(packet);
-
                 StartSSL();
-
                 packet.Clear();
                 packet.WriteInteger((int) connectionFlags,
                                     version.isAtLeast(4, 1, 0) ? 4 : 2);
@@ -297,39 +277,40 @@ namespace MySql.Data.MySqlClient
 
         private void StartSSL()
         {
-            RemoteCertificateValidationCallback sslValidateCallback;
-
-            sslValidateCallback = new RemoteCertificateValidationCallback(NoServerCheckValidation);
+            RemoteCertificateValidationCallback sslValidateCallback =
+                new RemoteCertificateValidationCallback(ServerCheckValidation);
             SslStream ss = new SslStream(baseStream, true, sslValidateCallback, null);
-            try
-            {
-                X509CertificateCollection certs = new X509CertificateCollection();
-                ss.AuthenticateAsClient(String.Empty, certs, SslProtocols.Default, false);
-                baseStream = ss;
-                stream = new MySqlStream(ss, encoding, false);
-                stream.SequenceByte = 2;
-            }
-            catch (Exception)
-            {
-                throw;
-            }
+            X509CertificateCollection certs = new X509CertificateCollection();
+            ss.AuthenticateAsClient(Settings.Server, certs, SslProtocols.Default, false);
+            baseStream = ss;
+            stream = new MySqlStream(ss, encoding, false);
+            stream.SequenceByte = 2;
+
         }
 
-/*        private static bool ServerCheckValidation(object sender, X509Certificate certificate,
+        private bool ServerCheckValidation(object sender, X509Certificate certificate,
                                                   X509Chain chain, SslPolicyErrors sslPolicyErrors)
         {
             if (sslPolicyErrors == SslPolicyErrors.None)
                 return true;
 
-            // Do not allow this client to communicate with unauthenticated servers.
+            if (Settings.SslMode == MySqlSslMode.Prefered ||
+                Settings.SslMode == MySqlSslMode.Required)
+            {
+                //Tolerate all certificate errors.
+                return true;
+            }
+
+            if (Settings.SslMode == MySqlSslMode.VerifyCA && 
+                sslPolicyErrors == SslPolicyErrors.RemoteCertificateNameMismatch)
+            {
+                // Tolerate name mismatch in certificate, if full validation is not requested.
+                return true;
+            }
+
             return false;
         }
-        */
-        private static bool NoServerCheckValidation(object sender, X509Certificate certificate,
-                                                    X509Chain chain, SslPolicyErrors sslPolicyErrors)
-        {
-            return true;
-        }
+
 
         #endregion
 
@@ -393,9 +374,13 @@ namespace MySql.Data.MySqlClient
                 flags |= ClientFlags.SECURE_CONNECTION;
 
             // if the server is capable of SSL and the user is requesting SSL
-            if ((serverCaps & ClientFlags.SSL) != 0 && connectionString.UseSSL)
+            if ((serverCaps & ClientFlags.SSL) != 0 && connectionString.SslMode != MySqlSslMode.None)
                 flags |= ClientFlags.SSL;
 
+            // if the server supports output parameters, then we do too
+            //if ((serverCaps & ClientFlags.PS_MULTI_RESULTS) != 0)
+                flags |= ClientFlags.PS_MULTI_RESULTS;
+
             connectionFlags = flags;
         }
 
@@ -465,6 +450,8 @@ namespace MySql.Data.MySqlClient
         /// </summary>
         public override void SendQuery(MySqlPacket queryPacket)
         {
+            lastInsertId = -1;
+            affectedRows = 0;
             if (Settings.Logging)
                 Logger.LogCommand(DBCmd.QUERY, encoding.GetString(
                     queryPacket.Buffer, 5, queryPacket.Length-5));
@@ -527,22 +514,35 @@ namespace MySql.Data.MySqlClient
             }
         }
 
+        public override void ExecuteDirect(string sql)
+        {
+            MySqlPacket p = new MySqlPacket(Encoding);
+            p.WriteString(sql);
+            SendQuery(p);
+            ReadResult();
+        }
+
         /// <summary>
         /// ReadResult will attempt to read a single result from the server.  Note that it is not 
         /// reading all the rows of the result set but simple determining what type of result it is
         /// and returning values appropriately.
         /// </summary>
-        /// <param name="affectedRows">Set to the number of rows affected in this result, 0 for selects</param>
-        /// <param name="lastInsertId">Set to the id of the row inserted by this result, 0 for non-inserts</param>
         /// <returns>Number of columns in the resultset, 0 for non-selects, -1 for no more resultsets</returns>
-        public override long ReadResult(ref ulong affectedRows, ref long lastInsertId)
+        public override long ReadResult()
         {
             // if there is not another query or resultset, then return -1
             if ((serverStatus & (ServerStatusFlags.AnotherQuery | ServerStatusFlags.MoreResults)) == 0)
                 return -1;
 
-            lastInsertId = -1;
-            packet = stream.ReadPacket();
+            try
+            {
+                packet = stream.ReadPacket();
+            }
+            catch (Exception)
+            {
+                serverStatus = 0;
+                throw;
+            }
 
             long fieldCount = packet.ReadFieldLength();
             if (fieldCount > 0)
@@ -553,14 +553,14 @@ namespace MySql.Data.MySqlClient
                 string filename = packet.ReadString();
                 SendFileToServer(filename);
 
-                return ReadResult(ref affectedRows, ref lastInsertId);
+                return ReadResult();
             }
 
             // the code to read last packet will set these server status vars 
             // again if necessary.
             serverStatus &= ~(ServerStatusFlags.AnotherQuery |
                               ServerStatusFlags.MoreResults);
-            affectedRows = (ulong) packet.ReadFieldLength();
+            affectedRows += (long)packet.ReadFieldLength();
             lastInsertId = packet.ReadFieldLength();
             if (version.isAtLeast(4, 1, 0))
             {
@@ -717,29 +717,13 @@ namespace MySql.Data.MySqlClient
                 colFlags = (ColumnFlags)packet.ReadInteger(2);
             else
                 colFlags = (ColumnFlags)packet.ReadByte();
-
-            field.SetTypeAndFlags(type, colFlags);
-
             field.Scale = (byte)packet.ReadByte();
-
-
             if (packet.HasMoreData)
             {
                 packet.ReadInteger(2); // reserved
             }
 
-            if (charSets != null && field.CharacterSetIndex != -1)
-            {
-                CharacterSet cs = CharSetMap.GetCharacterSet(Version, (string) charSets[field.CharacterSetIndex]);
-                // starting with 6.0.4 utf8 has a maxlen of 4 instead of 3.  The old
-                // 3 byte utf8 is utf8mb3
-                if (cs.name.ToLower(System.Globalization.CultureInfo.InvariantCulture) == "utf-8" &&
-                    Version.Major >= 6)
-                    field.MaxLength = 4;
-                else
-                    field.MaxLength = cs.byteCount;
-                field.Encoding = CharSetMap.GetEncoding(version, (string) charSets[field.CharacterSetIndex]);
-            }
+            field.SetTypeAndFlags(type, colFlags);
 
             return field;
         }
@@ -764,6 +748,8 @@ namespace MySql.Data.MySqlClient
 
         public void ExecuteStatement(MySqlPacket packetToExecute)
         {
+            lastInsertId = -1;
+            affectedRows = 0;
             packetToExecute.Buffer[4] = (byte)DBCmd.EXECUTE;
             ExecutePacket(packetToExecute);
             serverStatus |= ServerStatusFlags.AnotherQuery;
