@@ -238,7 +238,7 @@ namespace MySql.Data.MySqlClient
             string lowerType = row["DATA_TYPE"].ToString().ToLower();
             if (lowerType == "char" || lowerType == "varchar")
                 row["CHARACTER_MAXIMUM_LENGTH"] = dataLen;
-            else
+            else if (lowerType == "real" || lowerType == "decimal")
             {
                 string[] lenparts = dataLen.Split(new char[] {','});
                 row["NUMERIC_PRECISION"] = lenparts[0];
@@ -270,7 +270,8 @@ namespace MySql.Data.MySqlClient
             foreach (DataRow table in tables.Rows)
             {
                 string sql = String.Format("SHOW INDEX FROM `{0}`.`{1}`",
-                                           table["TABLE_SCHEMA"], table["TABLE_NAME"]);
+                    MySqlHelper.DoubleQuoteString((string)table["TABLE_SCHEMA"]), 
+                    MySqlHelper.DoubleQuoteString((string)table["TABLE_NAME"]));
                 MySqlDataAdapter da = new MySqlDataAdapter(sql, connection);
                 DataTable indexes = new DataTable();
                 da.Fill(indexes);
@@ -1018,14 +1019,8 @@ namespace MySql.Data.MySqlClient
                 for (int x = 0; x < restrictions.Length; x++)
                 {
                     string s = restrictions[x];
-                    if (s != null)
-                    {
-                        if (s.StartsWith("`"))
-                            s = s.Substring(1);
-                        if (s.EndsWith("`"))
-                            s = s.Substring(0, s.Length - 1);
-                        restrictions[x] = s;
-                    }
+                    if (s == null) continue;
+                    restrictions[x] = s.Trim('`');
                 }
             }
             return restrictions;
