@@ -34,7 +34,7 @@ using MySql.Data.MySqlClient.Properties;
 
 namespace MySql.Data.MySqlClient
 {
-	/// <include file='docs/mysqlcommand.xml' path='docs/ClassSummary/*'/>
+	/// <include file='docs/mysqlcommand.xml' path='docs/ClassSummary/*'/> 
 #if !CF
 	[System.Drawing.ToolboxBitmap(typeof(MySqlCommand), "MySqlClient.resources.command.bmp")]
 	[System.ComponentModel.DesignerCategory("Code")]
@@ -48,7 +48,6 @@ namespace MySql.Data.MySqlClient
 		long updatedRowCount;
 		UpdateRowSource updatedRowSource;
 		MySqlParameterCollection parameters;
-		private int cursorPageSize;
 		private IAsyncResult asyncResult;
 		private bool designTimeVisible;
 		internal Int64 lastInsertedId;
@@ -58,8 +57,6 @@ namespace MySql.Data.MySqlClient
         private bool resetSqlSelect;
         List<MySqlCommand> batch;
         private string batchableCommandText;
-        internal string parameterHash;
-        internal bool EFCrap;
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/ctor1/*'/>
 		public MySqlCommand()
@@ -124,15 +121,9 @@ namespace MySql.Data.MySqlClient
 					cmdText = cmdText.Substring(0, cmdText.Length - 14);
 					cmdText = cmdText + "() VALUES ()";
 				}
-
 			}
 		}
 
-		internal int UpdateCount
-		{
-			get { return (int)updatedRowCount; }
-		}
-
 		/// <include file='docs/mysqlcommand.xml' path='docs/CommandTimeout/*'/>
 #if !CF
 		[Category("Misc")]
@@ -250,6 +241,8 @@ namespace MySql.Data.MySqlClient
 		/// </remarks>
 		public override void Cancel()
 		{
+            if (canceled) return;
+
 			if (!connection.driver.Version.isAtLeast(5, 0, 0))
 				throw new NotSupportedException(Resources.CancelNotSupported);
 
@@ -295,9 +288,12 @@ namespace MySql.Data.MySqlClient
                 throw new InvalidOperationException("Connection must be valid and open.");
 
 			// Data readers have to be closed first
-			if (connection.Reader != null && cursorPageSize == 0)
+			if (connection.Reader != null)
 				throw new MySqlException("There is already an open DataReader associated with this Connection which must be closed first.");
 
+            if (CommandType == CommandType.TableDirect)
+                throw new InvalidOperationException(Resources.TableDirectNotSupported);
+
 			if (CommandType == CommandType.StoredProcedure && !connection.driver.Version.isAtLeast(5, 0, 0))
 				throw new MySqlException("Stored procedures are not supported on this version of MySQL");
 		}
@@ -305,24 +301,26 @@ namespace MySql.Data.MySqlClient
 		/// <include file='docs/mysqlcommand.xml' path='docs/ExecuteNonQuery/*'/>
 		public override int ExecuteNonQuery()
 		{
-			lastInsertedId = -1;
-			updatedRowCount = -1;
+            MySqlDataReader reader = null;
+            using (reader = ExecuteReader())
+            { 
+            }
 
-			MySqlDataReader reader = ExecuteReader();
-			if (reader != null)
-			{
-				reader.Close();
-				lastInsertedId = reader.InsertedId;
-				updatedRowCount = reader.RecordsAffected;
-			}
-			return (int)updatedRowCount;
+            return reader.RecordsAffected;
 		}
 
-		internal void Close()
+		internal void Close(MySqlDataReader reader)
 		{
 			if (statement != null)
-				statement.Close();
+				statement.Close(reader);
+            ResetSqlSelectLimit();
+        }
 
+        /// <summary>
+        /// Reset SQL_SELECT_LIMIT that could have been modified by CommandBehavior.
+        /// </summary>
+        internal void ResetSqlSelectLimit()
+        {
             // if we are supposed to reset the sql select limit, do that here
             if (resetSqlSelect)
                 new MySqlCommand("SET SQL_SELECT_LIMIT=-1", connection).ExecuteNonQuery();
@@ -338,6 +336,11 @@ namespace MySql.Data.MySqlClient
 		private void TimeoutExpired(object commandObject)
 		{
 			MySqlCommand cmd = (commandObject as MySqlCommand);
+            if (cmd == null)
+            {
+                Logger.LogWarning(Resources.TimeoutExpiredNullObject);
+                return;
+            }
 
             cmd.timedOut = true;
             try
@@ -346,10 +349,20 @@ namespace MySql.Data.MySqlClient
             }
             catch (Exception ex)
             {
-                // if something goes wrong, we log it and eat it.  There's really nothing
-                // else we can do.
+                // if something goes wrong, we log it and try to close the connection. 
+                // There's really nothing else we can do.
                 if (connection.Settings.Logging)
                     Logger.LogException(ex);
+
+                try
+                {
+                    cmd.Connection.Close();
+                }
+                catch (Exception ex1)
+                {
+                    if (connection.Settings.Logging)
+                        Logger.LogException(ex1);
+                }
             }
 		}
 
@@ -389,7 +402,7 @@ namespace MySql.Data.MySqlClient
 			}
 
             // stored procs are the only statement type that need do anything during resolve
-            statement.Resolve();
+            statement.Resolve(false);
 
             // Now that we have completed our resolve step, we can handle our
             // command behaviors
@@ -426,6 +439,12 @@ namespace MySql.Data.MySqlClient
             }
             catch (MySqlException ex)
             {
+                try
+                {
+                    ResetSqlSelectLimit();
+                }
+                catch (Exception) { }
+
                 // if we caught an exception because of a cancel, then just return null
                 if (ex.Number == 1317)
                 {
@@ -452,23 +471,13 @@ namespace MySql.Data.MySqlClient
             lastInsertedId = -1;
             object val = null;
 
-            MySqlDataReader reader = ExecuteReader();
-            if (reader == null) return null;
-
-            try
+            using (MySqlDataReader reader = ExecuteReader())
             {
+                if (reader == null) return null;
+
                 if (reader.Read())
                     val = reader.GetValue(0);
             }
-            finally
-            {
-                if (reader != null)
-                {
-                    reader.Close();
-                    lastInsertedId = reader.InsertedId;
-                }
-                reader = null;
-            }
 
             return val;
 		}
@@ -504,6 +513,7 @@ namespace MySql.Data.MySqlClient
 			else
 				statement = new PreparableStatement(this, CommandText);
 
+            statement.Resolve(true);
 			statement.Prepare();
 		}
 
@@ -524,7 +534,8 @@ namespace MySql.Data.MySqlClient
 
 		#region Async Methods
 
-		internal delegate void AsyncDelegate(int type, CommandBehavior behavior);
+		internal delegate object AsyncDelegate(int type, CommandBehavior behavior);
+        internal AsyncDelegate caller = null;
 		internal Exception thrownException;
 
 		private static string TrimSemicolons(string sql)
@@ -540,20 +551,20 @@ namespace MySql.Data.MySqlClient
 			return sb.ToString(start, end - start + 1);
 		}
 
-		internal void AsyncExecuteWrapper(int type, CommandBehavior behavior)
+		internal object AsyncExecuteWrapper(int type, CommandBehavior behavior)
 		{
 			thrownException = null;
 			try
 			{
-				if (type == 1)
-					ExecuteReader(behavior);
-				else
-					ExecuteNonQuery();
+                if (type == 1)
+                    return ExecuteReader(behavior);
+                return ExecuteNonQuery();
 			}
 			catch (Exception ex)
 			{
 				thrownException = ex;
 			}
+            return null;
 		}
 
 		/// <summary>
@@ -583,8 +594,11 @@ namespace MySql.Data.MySqlClient
 		/// the returned rows. </returns>
 		public IAsyncResult BeginExecuteReader(CommandBehavior behavior)
 		{
-			AsyncDelegate del = new AsyncDelegate(AsyncExecuteWrapper);
-			asyncResult = del.BeginInvoke(1, behavior, null, null);
+            if (caller != null)
+                throw new MySqlException(Resources.UnableToStartSecondAsyncOp);
+
+			caller = new AsyncDelegate(AsyncExecuteWrapper);
+			asyncResult = caller.BeginInvoke(1, behavior, null, null);
 			return asyncResult;
 		}
 
@@ -598,9 +612,11 @@ namespace MySql.Data.MySqlClient
 		public MySqlDataReader EndExecuteReader(IAsyncResult result)
 		{
 			result.AsyncWaitHandle.WaitOne();
-			if (thrownException != null)
-				throw thrownException;
-			return connection.Reader;
+            AsyncDelegate c = caller;
+            caller = null;
+            if (thrownException != null)
+                throw thrownException;
+            return (MySqlDataReader)c.EndInvoke(result);
 		}
 
 		/// <summary>
@@ -619,8 +635,11 @@ namespace MySql.Data.MySqlClient
 		/// which returns the number of affected rows. </returns>
 		public IAsyncResult BeginExecuteNonQuery(AsyncCallback callback, object stateObject)
 		{
-			AsyncDelegate del = new AsyncDelegate(AsyncExecuteWrapper);
-			asyncResult = del.BeginInvoke(2, CommandBehavior.Default, 
+            if (caller != null)
+                throw new MySqlException(Resources.UnableToStartSecondAsyncOp);
+
+            caller = new AsyncDelegate(AsyncExecuteWrapper);
+			asyncResult = caller.BeginInvoke(2, CommandBehavior.Default, 
 				callback, stateObject);
 			return asyncResult;
 		}
@@ -634,8 +653,11 @@ namespace MySql.Data.MySqlClient
 		/// which returns the number of affected rows. </returns>
 		public IAsyncResult BeginExecuteNonQuery()
 		{
-			AsyncDelegate del = new AsyncDelegate(AsyncExecuteWrapper);
-			asyncResult = del.BeginInvoke(2, CommandBehavior.Default, null, null);
+            if (caller != null)
+                throw new MySqlException(Resources.UnableToStartSecondAsyncOp);
+
+            caller = new AsyncDelegate(AsyncExecuteWrapper);
+			asyncResult = caller.BeginInvoke(2, CommandBehavior.Default, null, null);
 			return asyncResult;
 		}
 
@@ -648,9 +670,11 @@ namespace MySql.Data.MySqlClient
 		public int EndExecuteNonQuery(IAsyncResult asyncResult)
 		{
 			asyncResult.AsyncWaitHandle.WaitOne();
-			if (thrownException != null)
-				throw thrownException;
-			return (int)updatedRowCount;
+            AsyncDelegate c = caller;
+            caller = null;
+            if (thrownException != null)
+                throw thrownException;
+            return (int)c.EndInvoke(asyncResult);
 		}
 
 		#endregion
@@ -717,25 +741,32 @@ namespace MySql.Data.MySqlClient
 		#endregion
 
 		#region ICloneable
+
 		/// <summary>
 		/// Creates a clone of this MySqlCommand object.  CommandText, Connection, and Transaction properties
 		/// are included as well as the entire parameter list.
 		/// </summary>
 		/// <returns>The cloned MySqlCommand object</returns>
-		object ICloneable.Clone()
+		public MySqlCommand Clone()
 		{
 			MySqlCommand clone = new MySqlCommand(cmdText, connection, curTransaction);
             clone.CommandType = CommandType;
             clone.CommandTimeout = CommandTimeout;
             clone.batchableCommandText = batchableCommandText;
-            clone.EFCrap = EFCrap;
+            clone.UpdatedRowSource = UpdatedRowSource;
 
 			foreach (MySqlParameter p in parameters)
 			{
-				clone.Parameters.Add((p as ICloneable).Clone());
+				clone.Parameters.Add(p.Clone());
 			}
 			return clone;
 		}
+
+        object ICloneable.Clone()
+        {
+            return this.Clone();
+        }
+
 		#endregion
 
         #region Batching support
