@@ -370,12 +370,20 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-        internal void Serialize(MySqlPacket packet, bool binary)
+        internal void Serialize(MySqlPacket packet, bool binary, MySqlConnectionStringBuilder settings)
         {
             if (!binary && (paramValue == null || paramValue == DBNull.Value))
                 packet.WriteStringNoNull("NULL");
             else
+            {
+                if (ValueObject.MySqlDbType == MySqlDbType.Guid)
+                {
+                    MySqlGuid g = (MySqlGuid)ValueObject;
+                    g.OldGuids = settings.OldGuids;
+                    valueObject = g;
+                }
                 ValueObject.WriteValue(packet, binary, paramValue, size);
+            }
         }
 
         private void SetMySqlDbType(MySqlDbType mysql_dbtype)
@@ -600,7 +608,7 @@ namespace MySql.Data.MySqlClient
 
         #region ICloneable
 
-        object ICloneable.Clone()
+        public MySqlParameter Clone()
         {
             MySqlParameter clone = new MySqlParameter(paramName, mySqlDbType, direction,
                 sourceColumn, sourceVersion, paramValue);
@@ -609,6 +617,11 @@ namespace MySql.Data.MySqlClient
             return clone;
         }
 
+        object ICloneable.Clone()
+        {
+            return this.Clone();
+        }
+
         #endregion
 
         /// <summary>
