@@ -608,7 +608,7 @@
  
                  { 
  
-                         s t r i n g   c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ; r e s p e c t   b i n a r y   f l a g s = f a l s e " ; 
+                         s t r i n g   c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ; r e s p e c t   b i n a r y   f l a g s = t r u e " ; 
  
                          u s i n g   ( M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ) 
  
@@ -626,6 +626,30 @@
  
                                  d a . F i l l ( d t ) ; 
  
+                                 A s s e r t . I s T r u e ( d t . R o w s [ 0 ] [ 0 ]   i s   b y t e [ ] ) ; 
+ 
+                         } 
+ 
+                         c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ; r e s p e c t   b i n a r y   f l a g s = f a l s e " ; 
+ 
+                         u s i n g   ( M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ) 
+ 
+                         { 
+ 
+                                 c . O p e n ( ) ; 
+ 
+ 
+ 
+                                 M y S q l D a t a A d a p t e r   d a   =   n e w   M y S q l D a t a A d a p t e r ( 
+ 
+                                         " S E L E C T   C O N C A T ( ' T r  d g  r d s v  g e n ' ,   1 ) " ,   c ) ; 
+ 
+                                 D a t a T a b l e   d t   =   n e w   D a t a T a b l e ( ) ; 
+ 
+                                 d a . F i l l ( d t ) ; 
+ 
+                                 A s s e r t . I s T r u e ( d t . R o w s [ 0 ] [ 0 ]   i s   s t r i n g ) ; 
+ 
                                  A s s e r t . A r e E q u a l ( " T r  d g  r d s v  g e n 1 " ,   d t . R o w s [ 0 ] [ 0 ] ) ; 
  
                          } 
