@@ -128,9 +128,9 @@ namespace MySql.Data.MySqlClient.Tests
 		[Test]
 		public void CloneCommand() 
 		{
-			IDbCommand cmd = new MySqlCommand();
-			IDbCommand cmd2 = ((ICloneable)cmd).Clone() as IDbCommand;
-			cmd2.ToString();
+			MySqlCommand cmd = new MySqlCommand();
+            MySqlCommand newCommand = cmd.Clone();
+            IDbCommand newCommand2 = (IDbCommand)(cmd as ICloneable).Clone();
 		}
 
 		[Test]
@@ -363,6 +363,27 @@ namespace MySql.Data.MySqlClient.Tests
             }
         }
 
+        /// <summary>
+        /// Bug #45941	SQL-Injection attack
+        /// </summary>
+        [Test]
+        public void SqlInjection1()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test(name VARCHAR(100)) ENGINE=MyISAM DEFAULT CHARSET=utf8");
+            execSQL("INSERT INTO Test VALUES ('name1'), ('name2'), ('name3')");
+
+            MySqlCommand cnt = new MySqlCommand("SELECT COUNT(*) FROM Test", conn);
+            Int64 count = (Int64)cnt.ExecuteScalar();
+
+            MySqlCommand cmd = new MySqlCommand("DELETE FROM Test WHERE name=?name", conn);
+            cmd.Parameters.Add("?name", MySqlDbType.VarChar);
+            cmd.Parameters[0].Value = "\u2032 OR 1=1;-- --";
+            cmd.ExecuteNonQuery();
+
+            Assert.AreEqual(count, (Int64)cnt.ExecuteScalar());
+        }
+
         /// <summary>
         /// Bug #44194	ExecuteNonQuery for update commands does not match actual rows updated
         /// </summary>
@@ -385,6 +406,36 @@ namespace MySql.Data.MySqlClient.Tests
                 Assert.AreEqual(0, cmd.ExecuteNonQuery());
             }
         }
+
+        /// <summary>
+        /// Bug #45502 error if "Allow Batch=False" 
+        /// </summary>
+        [Test]
+        public void DontAllowBatching()
+        {
+            string connStr = GetConnectionString(true) + ";allow batch=false;character set=utf8";
+            using (MySqlConnection c = new MySqlConnection(connStr))
+            {
+                c.Open();
+                MySqlCommand cmd = new MySqlCommand("SELECT 1", c);
+                cmd.ExecuteScalar();
+            }
+        }
+
+        [Test]
+        public void TableCommandType()
+        {
+            MySqlCommand cmd = new MySqlCommand("Mytable", conn);
+            cmd.CommandType = CommandType.TableDirect;
+            try
+            {
+                cmd.ExecuteReader();
+                Assert.Fail("This should have failed");
+            }
+            catch (InvalidOperationException ex)
+            {
+            }
+        }
     }
 
 
