@@ -72,7 +72,43 @@ namespace MySql.Data.MySqlClient.Tests
 			}
 		}
 
-		[Test]
+        /// <summary>
+        /// Bug#46205 - tinyint as boolean does not work for utf8 default database character set.
+        /// </summary>
+        ///<remarks>
+        /// Original bug occured only with mysqld started with --default-character-set=utf8.
+        /// It does not seem  possible to reproduce the original buggy behaviorotherwise
+        /// Neither "set global character_set_server = utf8" , nor  "create table /database with character set "
+        /// were sufficient.
+        ///</remarks>
+        [Test]
+        public void TreatTinyAsBool()
+        {
+            if (version < new Version(4, 1)) return;
+            execSQL("CREATE TABLE Test2(i TINYINT(1))");
+            execSQL("INSERT INTO Test2 VALUES(1)");
+            execSQL("INSERT INTO Test2 VALUES(0)");
+            execSQL("INSERT INTO Test2 VALUES(2)");
+            MySqlConnectionStringBuilder builder = new MySqlConnectionStringBuilder(conn.ConnectionString);
+            Assert.IsTrue(builder.TreatTinyAsBoolean);
+
+            MySqlCommand cmd = new MySqlCommand("SELECT * from Test2", conn);
+            using (MySqlDataReader reader = cmd.ExecuteReader())
+            {
+                bool b;
+                Assert.IsTrue(reader.Read());
+                b = (bool)reader.GetValue(0);
+                Assert.IsTrue(b);
+                Assert.IsTrue(reader.Read());
+                b = (bool)reader.GetValue(0);
+                Assert.IsFalse(b);
+                Assert.IsTrue(reader.Read());
+                b = (bool)reader.GetValue(0);
+                Assert.IsTrue(b);
+            }
+        }
+
+        [Test]
 		public void TestFloat() 
 		{
 			InternalTestFloats(false);
@@ -650,37 +686,42 @@ namespace MySql.Data.MySqlClient.Tests
             execSQL("DROP TABLE IF EXISTS Test");
             execSQL("CREATE TABLE Test (id INT, g BINARY(16), c VARBINARY(16), c1 BINARY(255))");
 
-            Guid g = Guid.NewGuid();
-            byte[] bytes = g.ToByteArray();
-
-            MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES (1, @g, @c, @c1)", conn);
-            cmd.Parameters.AddWithValue("@g", bytes);
-            cmd.Parameters.AddWithValue("@c", bytes);
-            cmd.Parameters.AddWithValue("@c1", g.ToString());
-            cmd.ExecuteNonQuery();
-
-            MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", conn);
-            DataTable dt = new DataTable();
-            da.Fill(dt);
-            Assert.IsTrue(dt.Rows[0][1] is Guid);
-            Assert.IsTrue(dt.Rows[0][2] is byte[]);
-            Assert.IsTrue(dt.Rows[0][3] is byte[]);
-
-            Assert.AreEqual(g, dt.Rows[0][1]);
-
-            string s = BitConverter.ToString(bytes);
-
-            s = s.Replace("-", "");
-            string sql = String.Format("TRUNCATE TABLE Test;INSERT INTO Test VALUES(1,0x{0},NULL,NULL)", s);
-            execSQL(sql);
-
-            cmd.CommandText = "SELECT * FROM Test";
-            cmd.Parameters.Clear();
-            using (MySqlDataReader reader = cmd.ExecuteReader())
+            string connStr = GetConnectionString(true) + ";old guids=true";
+            using (MySqlConnection c = new MySqlConnection(connStr))
             {
-                reader.Read();
-                Guid g1 = reader.GetGuid(1);
-                Assert.AreEqual(g, g1);
+                c.Open();
+                Guid g = Guid.NewGuid();
+                byte[] bytes = g.ToByteArray();
+
+                MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES (1, @g, @c, @c1)", c);
+                cmd.Parameters.AddWithValue("@g", bytes);
+                cmd.Parameters.AddWithValue("@c", bytes);
+                cmd.Parameters.AddWithValue("@c1", g.ToString());
+                cmd.ExecuteNonQuery();
+
+                MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", c);
+                DataTable dt = new DataTable();
+                da.Fill(dt);
+                Assert.IsTrue(dt.Rows[0][1] is Guid);
+                Assert.IsTrue(dt.Rows[0][2] is byte[]);
+                Assert.IsTrue(dt.Rows[0][3] is byte[]);
+
+                Assert.AreEqual(g, dt.Rows[0][1]);
+
+                string s = BitConverter.ToString(bytes);
+
+                s = s.Replace("-", "");
+                string sql = String.Format("TRUNCATE TABLE Test;INSERT INTO Test VALUES(1,0x{0},NULL,NULL)", s);
+                execSQL(sql);
+
+                cmd.CommandText = "SELECT * FROM Test";
+                cmd.Parameters.Clear();
+                using (MySqlDataReader reader = cmd.ExecuteReader())
+                {
+                    reader.Read();
+                    Guid g1 = reader.GetGuid(1);
+                    Assert.AreEqual(g, g1);
+                }
             }
         }
 
@@ -813,18 +854,23 @@ namespace MySql.Data.MySqlClient.Tests
         {
             execSQL("CREATE TABLE Test(id INT, g BINARY(16))");
 
-            Guid guid = Guid.NewGuid();
-            MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES(1, @g)", conn);
-            cmd.Parameters.Add(new MySqlParameter("@g", MySqlDbType.Guid));
-            cmd.Parameters[0].Value = guid;
-            cmd.ExecuteNonQuery();
+            string connStr = GetConnectionString(true) + ";old guids=true";
+            using (MySqlConnection c = new MySqlConnection(connStr))
+            {
+                c.Open();
+                Guid guid = Guid.NewGuid();
+                MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES(1, @g)", c);
+                cmd.Parameters.Add(new MySqlParameter("@g", MySqlDbType.Guid));
+                cmd.Parameters[0].Value = guid;
+                cmd.ExecuteNonQuery();
 
-            DataTable dt = new DataTable();
-            MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", conn);
-            da.Fill(dt);
-            Assert.AreEqual(1, dt.Rows.Count);
-            Assert.AreEqual(1, dt.Rows[0]["id"]);
-            Assert.AreEqual(guid, dt.Rows[0]["g"]);
+                DataTable dt = new DataTable();
+                MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", c);
+                da.Fill(dt);
+                Assert.AreEqual(1, dt.Rows.Count);
+                Assert.AreEqual(1, dt.Rows[0]["id"]);
+                Assert.AreEqual(guid, dt.Rows[0]["g"]);
+            }
         }
 
         /// <summary>
@@ -836,26 +882,157 @@ namespace MySql.Data.MySqlClient.Tests
             execSQL("DROP TABLE IF EXISTS Test");
             execSQL("CREATE TABLE Test (id INT, guid BINARY(16))");
 
-            Guid g = new Guid("32A48AC5-285A-46c6-A0D4-158E6E39729C");
-            MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES (1, ?guid)", conn);
-            MySqlParameter p = new MySqlParameter();
-            p.ParameterName = "guid";
-            p.Value = Guid.NewGuid();
-            cmd.Parameters.Add(p);
+            string connStr = GetConnectionString(true) + ";old guids=true";
+            using (MySqlConnection c = new MySqlConnection(connStr))
+            {
+                c.Open();
+
+                Guid g = new Guid("32A48AC5-285A-46c6-A0D4-158E6E39729C");
+                MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES (1, ?guid)", c);
+                //MySqlParameter p = new MySqlParameter();
+                //p.ParameterName = "guid";
+                //p.Value = Guid.NewGuid();
+                cmd.Parameters.AddWithValue("guid", Guid.NewGuid());
+                cmd.ExecuteNonQuery();
+
+                cmd.CommandText = "SELECT * FROM Test";
+                cmd.Parameters.Clear();
+                using (MySqlDataReader reader = cmd.ExecuteReader())
+                {
+                    reader.Read();
+
+                    object o = reader.GetValue(1);
+                    Assert.IsTrue(o is Guid);
+
+                    byte[] bytes = new byte[16];
+                    long size = reader.GetBytes(1, 0, bytes, 0, 16);
+                    Assert.AreEqual(16, size);
+                }
+            }
+        }
+
+        [Test]
+        public void ReadingUUIDAsGuid()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id INT, guid CHAR(36))");
+            execSQL("INSERT INTO Test VALUES (1, UUID())");
+
+            MySqlCommand cmd = new MySqlCommand("SELECT CONCAT('A', guid) FROM Test", conn);
+            string serverGuidStr = cmd.ExecuteScalar().ToString().Substring(1);
+            Guid serverGuid = new Guid(serverGuidStr);
+
+            cmd.CommandText = "SELECT guid FROM Test";
+            Guid g = (Guid)cmd.ExecuteScalar();
+            Assert.AreEqual(serverGuid, g);
+        }
+
+        [Test]
+        public void NewGuidType()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id INT, guid CHAR(36))");
+
+            Guid g = Guid.NewGuid();
+            MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES(1, @g)", conn);
+            cmd.Parameters.AddWithValue("@g", g);
             cmd.ExecuteNonQuery();
 
-            cmd.CommandText = "SELECT * FROM Test";
-            cmd.Parameters.Clear();
+            cmd.CommandText = "SELECT guid FROM Test";
+            Guid readG = (Guid)cmd.ExecuteScalar();
+            Assert.AreEqual(g, readG);
+        }
+
+        /// <summary>
+        /// Bug #47928 Old Guids=true setting is lost after null value is
+        /// encountered in a Binary(16) 
+        /// </summary>
+        [Test]
+        public void OldGuidsWithNull()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id INT, guid BINARY(16))");
+
+            string connStr = GetConnectionString(true) + ";old guids=true";
+            using (MySqlConnection c = new MySqlConnection(connStr))
+            {
+                c.Open();
+
+                MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES (1, ?guid)", c);
+                cmd.Parameters.AddWithValue("guid", Guid.NewGuid());
+                cmd.ExecuteNonQuery();
+
+                cmd.Parameters["guid"].Value = null;
+                cmd.ExecuteNonQuery();
+                cmd.Parameters["guid"].Value = Guid.NewGuid();
+                cmd.ExecuteNonQuery();
+
+                cmd.CommandText = "SELECT guid FROM Test";
+                using (MySqlDataReader reader = cmd.ExecuteReader())
+                {
+                    //In Bug #47928, following loop will crash after encountering
+                    // null value.
+                    while (reader.Read())
+                    {
+                        object o = reader.GetValue(0);
+                    }
+                }
+            }
+        }
+
+        /// <summary>
+        /// Bug #47985	UTF-8 String Length Issue (guids etc)
+        /// </summary>
+        [Test]
+        public void UTF8Char12AsGuid()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id INT, name CHAR(12) CHARSET utf8)");
+            execSQL("INSERT INTO Test VALUES (1, 'Name')");
+
+            string connStr = GetConnectionString(true) + ";charset=utf8";
+            using (MySqlConnection c = new MySqlConnection(connStr))
+            {
+                c.Open();
+
+                MySqlCommand cmd = new MySqlCommand("SELECT * FROM Test", c);
+                using (MySqlDataReader reader = cmd.ExecuteReader())
+                {
+                    reader.Read();
+                    string s = reader.GetString(1);
+                    Assert.AreEqual("Name", s);
+                }
+            }
+        }
+
+        /// <summary>
+        /// Bug #48100	Impossible to retrieve decimal value if it doesn't fit into .Net System.Decimal
+        /// </summary>
+        [Test]
+        public void MySqlDecimal()
+        {
+            if (Version < new Version(5, 0)) return;
+
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id INT, dec1 DECIMAL(36,2))");
+            execSQL("INSERT INTO Test VALUES (1, 9999999999999999999999999999999999.99)");
+
+            MySqlCommand cmd = new MySqlCommand("SELECT * FROM Test", conn);
             using (MySqlDataReader reader = cmd.ExecuteReader())
             {
                 reader.Read();
-
-                object o = reader.GetValue(1);
-                Assert.IsTrue(o is Guid);
-
-                byte[] bytes = new byte[16];
-                long size = reader.GetBytes(1, 0, bytes, 0, 16);
-                Assert.AreEqual(16, size);
+                MySqlDecimal dec = reader.GetMySqlDecimal(1);
+                string s = dec.ToString();
+                Assert.AreEqual(9999999999999999999999999999999999.99, dec.ToDouble());
+                Assert.AreEqual("9999999999999999999999999999999999.99", dec.ToString());
+                try
+                {
+                    decimal d = dec.Value;
+                    Assert.Fail("this should have failed");
+                }
+                catch (Exception ex) 
+                {
+                }
             }
         }
     }
