@@ -237,6 +237,23 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("TINYINT", dt.Rows[3]["DATA_TYPE"].ToString().ToUpper());
 		}
 
+        /// <summary> 
+        /// Bug #46270 connection.GetSchema("Columns") fails on MySQL 4.1  
+        /// </summary> 
+        [Test]
+        public void EnumAndSetColumns()
+        {
+            execSQL("DROP TABLE IF EXISTS test");
+            execSQL("CREATE TABLE test (col1 set('A','B','C'), col2 enum('A','B','C'))");
+
+            DataTable dt = conn.GetSchema("Columns", new string[] { null, null, "test", null });
+            Assert.AreEqual(2, dt.Rows.Count);
+            Assert.AreEqual("set", dt.Rows[0]["DATA_TYPE"]);
+            Assert.AreEqual("enum", dt.Rows[1]["DATA_TYPE"]);
+            Assert.AreEqual("set('A','B','C')", dt.Rows[0]["COLUMN_TYPE"]);
+            Assert.AreEqual("enum('A','B','C')", dt.Rows[1]["COLUMN_TYPE"]);
+        } 
+
 		[Test]
 		public void Procedures()
 		{
@@ -305,13 +322,17 @@ namespace MySql.Data.MySqlClient.Tests
             Assert.AreEqual(false, dt.Rows[0]["PRIMARY"]);
             Assert.AreEqual(true, dt.Rows[0]["UNIQUE"]);
 
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (id int, name varchar(50), " +
-				"KEY key2 (name))");
+            /// <summary> 
+            /// Bug #48101	MySqlConnection.GetSchema on "Indexes" throws when there's a table named "b`a`d" 
+            /// </summary> 
+            execSQL("DROP TABLE IF EXISTS test");
+            execSQL(@"CREATE TABLE `te``s``t` (id int, name varchar(50), " +
+                "KEY key2 (name))");
 
-			dt = conn.GetSchema("Indexes", restrictions);
+            restrictions[2] = "te`s`t"; 
+            dt = conn.GetSchema("Indexes", restrictions);
 			Assert.AreEqual(1, dt.Rows.Count);
-			Assert.AreEqual("test", dt.Rows[0]["TABLE_NAME"]);
+            Assert.AreEqual("te`s`t", dt.Rows[0]["TABLE_NAME"]);
 			Assert.AreEqual("key2", dt.Rows[0]["INDEX_NAME"]);
 			Assert.AreEqual(false, dt.Rows[0]["PRIMARY"]);
 			Assert.AreEqual(false, dt.Rows[0]["UNIQUE"]);
