@@ -70,23 +70,13 @@
  
  	 { 
  
-                 p u b l i c   P o o l i n g T e s t s ( ) 
- 
-                 { 
- 
-                         p o o l i n g   =   t r u e ; 
- 
-                 } 
- 
- 
- 
  	 	 [ T e s t ] 
  
  	 	 p u b l i c   v o i d   C o n n e c t i o n ( ) 
  
  	 	 { 
  
- 	 	 	 s t r i n g   c o n n S t r   =   c o n n . C o n n e c t i o n S t r i n g   +   " ; p o o l i n g = t r u e " ; 
+                         s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( ) ; 
  
  
  
@@ -180,9 +170,9 @@
  
  	 	 p u b l i c   v o i d   O p e n K i l l e d ( ) 
  
- 	 	 { 
+ 	 	 {   
  
- 	 	 	 s t r i n g   c o n n S t r   =   c o n n . C o n n e c t i o n S t r i n g   +   " ; p o o l i n g = t r u e ; m i n   p o o l   s i z e = 1 ;   m a x   p o o l   s i z e = 1 " ; 
+                         s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( )   +   " ; m i n   p o o l   s i z e = 1 ;   m a x   p o o l   s i z e = 1 " ; 
  
  	 	 	 M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
  
@@ -226,7 +216,7 @@
  
  	 	 	 / /   n o w   c r e a t e   a   n e w   c o n n e c t i o n   s t r i n g   o n l y   a l l o w i n g   1   c o n n e c t i o n   i n   t h e   p o o l 
  
- 	 	 	 s t r i n g   c o n n S t r   =   c o n n . C o n n e c t i o n S t r i n g   +   " ; m a x   p o o l   s i z e = 1 " ; 
+                         s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( )   +   " ; c o n n e c t   t i m e o u t = 2 ; m a x   p o o l   s i z e = 1 " ; 
  
  
  
@@ -284,7 +274,9 @@
  
  	 	 	 c o n n e c t i o n . O p e n ( ) ; 
  
- 	 	 	 c o n n e c t i o n . C l o s e ( ) ; 
+                         K i l l C o n n e c t i o n ( c o n n e c t i o n ) ; 
+ 
+                         c o n n e c t i o n . C l o s e ( ) ; 
  
  	 	 } 
  
@@ -296,23 +288,39 @@
  
  	 	 { 
  
- 	 	 	 e x e c S Q L ( " S E T   @ t e s t v a r = ' 5 ' " ) ; 
+                         s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( ) ; 
  
- 	 	 	 M y S q l C o m m a n d   c m d   =   n e w   M y S q l C o m m a n d ( " S E L E C T   @ t e s t v a r " ,   c o n n ) ; 
+                         u s i n g   ( M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ) 
  
- 	 	 	 o b j e c t   v a r   =   c m d . E x e c u t e S c a l a r ( ) ; 
+                         { 
  
- 	 	 	 A s s e r t . A r e E q u a l ( " 5 " ,   v a r ) ; 
+                                 c . O p e n ( ) ; 
  
- 	 	 	 c o n n . C l o s e ( ) ; 
+                                 M y S q l C o m m a n d   c m d   =   n e w   M y S q l C o m m a n d ( " S E T   @ t e s t v a r = ' 5 ' " ,   c ) ; 
+ 
+                                 c m d . E x e c u t e N o n Q u e r y ( ) ; 
  
  
  
- 	 	 	 c o n n . O p e n ( ) ; 
+                                 c m d . C o m m a n d T e x t   =   " S E L E C T   @ t e s t v a r " ; 
  
- 	 	 	 o b j e c t   v a r 2   =   c m d . E x e c u t e S c a l a r ( ) ; 
+                                 o b j e c t   v a r   =   c m d . E x e c u t e S c a l a r ( ) ; 
  
- 	 	 	 A s s e r t . A r e E q u a l ( D B N u l l . V a l u e ,   v a r 2 ) ; 
+                                 A s s e r t . A r e E q u a l ( " 5 " ,   v a r ) ; 
+ 
+                                 c . C l o s e ( ) ; 
+ 
+ 
+ 
+                                 c . O p e n ( ) ; 
+ 
+         	 	 	 o b j e c t   v a r 2   =   c m d . E x e c u t e S c a l a r ( ) ; 
+ 
+ 	         	 	 A s s e r t . A r e E q u a l ( D B N u l l . V a l u e ,   v a r 2 ) ; 
+ 
+                                 K i l l C o n n e c t i o n ( c ) ; 
+ 
+                         } 
  
  	 	 } 
  
@@ -343,7 +351,7 @@
  
  
  
- 	 	 	 s t r i n g   c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ; c h a r s e t = u t f 8 " ; 
+ 	 	 	 s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( )   +   " ; c h a r s e t = u t f 8 " ; 
  
                          u s i n g   ( M y S q l C o n n e c t i o n   c o n   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ) 
  
@@ -382,6 +390,8 @@
  
  
  
+                                 K i l l C o n n e c t i o n ( c o n ) ; 
+ 
                                  c o n . C l o s e ( ) ; 
  
                                  A s s e r t . A r e E q u a l ( f i r s t S ,   s e c o n d S ) ; 
@@ -424,7 +434,7 @@
  
  	 	 { 
  
- 	 	 	 s t r i n g   c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ; m a x   p o o l   s i z e = 1 " ; 
+ 	 	 	 s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( )   +   " ; m a x   p o o l   s i z e = 1 " ; 
  
  	 	 	 M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
  
@@ -446,8 +456,12 @@
  
                                  c 2 . O p e n ( ) ; 
  
+                                 K i l l C o n n e c t i o n ( c 2 ) ; 
+ 
                          } 
  
+                         c . C l o s e ( ) ; 
+ 
  	 	 } 
  
  
@@ -482,7 +496,7 @@
  
  
  
-                         s t r i n g   c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ; l o g g i n g = t r u e " ; 
+                         s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( ) ; 
  
  
  
@@ -528,6 +542,12 @@
  
  	 	 	 } 
  
+                         M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
+ 
+                         c . O p e n ( ) ; 
+ 
+                         K i l l C o n n e c t i o n ( c ) ; 
+ 
  	 	 } 
  
  
@@ -584,7 +604,7 @@
  
  	 	 { 
  
- 	 	 	 s t r i n g   c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ;   m i n   p o o l   s i z e = 1 0 " ; 
+ 	 	 	 s t r i n g   c o n n S t r   =   G e t P o o l i n g C o n n e c t i o n S t r i n g ( )   +   " ; m i n   p o o l   s i z e = 1 0 " ; 
  
  	 	 	 M y S q l C o n n e c t i o n [ ]   c o n n e c t i o n s   =   n e w   M y S q l C o n n e c t i o n [ 1 0 ] ; 
  
