@@ -39,8 +39,7 @@ namespace MySql.Data.MySqlClient.Tests
 
         public StoredProcedure()
         {
-            pooling = false;
-            csAdditions = ";procedure cache size=0;logging=true;";
+            csAdditions = ";procedure cache size=0;";
         }
 
         /// <summary>
@@ -867,7 +866,7 @@ namespace MySql.Data.MySqlClient.Tests
             execSQL("CREATE PROCEDURE spTest(id int, OUT outid int, INOUT inoutid int) " +
                 "BEGIN SET outid=id+inoutid; SET inoutid=inoutid+id; END");
 
-            string s = GetConnectionStringEx("testuser", "testuser", true);
+            string s = GetConnectionString("testuser", "testuser", true);
             MySqlConnection c = new MySqlConnection(s);
             c.Open();
 
@@ -1218,28 +1217,31 @@ namespace MySql.Data.MySqlClient.Tests
 
             execSQL("CREATE PROCEDURE spTest(myparam decimal  (8,2)) BEGIN SELECT 1; END");
 
-            string connStr = GetConnectionString(true);
-            connStr = connStr.Replace("use procedure bodies=false", "");
-            using (MySqlConnection c = new MySqlConnection(connStr))
-            {
-                c.Open();
-
-                MySqlCommand cmd = new MySqlCommand("spTest", c);
-                cmd.Parameters.Add("@myparam", MySqlDbType.Decimal).Value = 20;
-                cmd.CommandType = CommandType.StoredProcedure;
-                object o = cmd.ExecuteScalar();
-                Assert.AreEqual(1, o);
-            }
+            MySqlCommand cmd = new MySqlCommand("spTest", conn);
+            cmd.Parameters.Add("@myparam", MySqlDbType.Decimal).Value = 20;
+            cmd.CommandType = CommandType.StoredProcedure;
+            object o = cmd.ExecuteScalar();
+            Assert.AreEqual(1, o);
         }
 
-        [Test]
-        public void ParametersInReverseOrder()
+        private void ParametersInReverseOrderInternal(bool isOwner)
         {
-            if (Version.Major != 5) return;
+            if (Version.Major < 5) return;
 
-            execSQL(@"CREATE PROCEDURE spTest(IN p_1 VARCHAR(5), IN p_2 VARCHAR(5))
-                        BEGIN SELECT p_1 AS P1, p_2 AS P2; END");
-            MySqlCommand cmd = new MySqlCommand("spTest", conn);
+            string sql = @"CREATE PROCEDURE spTest(IN p_1 VARCHAR(5), IN p_2 VARCHAR(5))
+                        BEGIN SELECT p_1 AS P1, p_2 AS P2; END";
+            string spName = "spTest";
+            if (!isOwner)
+            {
+                rootConn.ChangeDatabase(database1);
+                suExecSQL(sql);
+                rootConn.ChangeDatabase(database0);
+                spName = database1 + ".spTest";
+            }
+            else
+                execSQL(sql);
+
+            MySqlCommand cmd = new MySqlCommand(spName, conn);
             cmd.CommandType = CommandType.StoredProcedure;
             cmd.CommandTimeout = 0;
             cmd.Parameters.AddWithValue("?p_2", ("World"));
@@ -1254,8 +1256,8 @@ namespace MySql.Data.MySqlClient.Tests
             string s = GetConnectionString(true);
             if (s.IndexOf("use procedure bodies=false") != -1)
             {
-                Assert.AreEqual("World", dt.Rows[0]["P1"]);
-                Assert.AreEqual("Hello", dt.Rows[0]["P2"]);
+                Assert.AreEqual("World", dt.Rows[0][0]);
+                Assert.AreEqual("Hello", dt.Rows[0][1]);
             }
             else
             {
@@ -1263,5 +1265,32 @@ namespace MySql.Data.MySqlClient.Tests
                 Assert.AreEqual("World", dt.Rows[0]["P2"]);
             }
         }
+
+        [Test]
+        public void ParametersInReverseOrderNotOwner()
+        {
+            ParametersInReverseOrderInternal(false);
+        }
+
+        [Test]
+        public void ParametersInReverseOrderOwner()
+        {
+            ParametersInReverseOrderInternal(true);
+        }
+
+        [Test]
+        public void DeriveParameters()
+        {
+            if (Version < new Version(5, 0)) return;
+            if (Version > new Version(6, 0, 6)) return;
+
+            execSQL(@"CREATE  PROCEDURE spTest (id INT, name VARCHAR(20))
+                    BEGIN SELECT name; END");
+
+            MySqlCommand cmd = new MySqlCommand("spTest", conn);
+            cmd.CommandType = CommandType.StoredProcedure;
+            MySqlCommandBuilder.DeriveParameters(cmd);
+            Assert.AreEqual(2, cmd.Parameters.Count);
+        }
     }
 }
