@@ -39,7 +39,7 @@ namespace MySql.Web.Common
     /// </summary>
     public static class SchemaManager
     {
-        private const int schemaVersion = 4;
+        private const int schemaVersion = 6;
 
         /// <summary>
         /// Gets the most recent version of the schema.
@@ -57,7 +57,7 @@ namespace MySql.Web.Common
                 int ver = GetSchemaVersion(connectionString);
                 if (ver == Version) return;
 
-                if (config["autogenerateschema"] == "true")
+                if (String.Compare(config["autogenerateschema"], "true", true) == 0)
                     UpgradeToCurrent(connectionString, ver);
                 else
                     throw new ProviderException(Resources.MissingOrWrongSchema);
